#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MAX_ITERATIONS 1000
#define EPSILON 0.001

void jacobi(double **A, double *b, double *x, int n) {
    int i, j, k;
    double *new_x = malloc(n * sizeof(double));
    double sum;

    for (k = 0; k < MAX_ITERATIONS; k++) {
        for (i = 0; i < n; i++) {
            sum = 0.0;
            for (j = 0; j < n; j++) {
                if (j != i) {
                    sum += A[i][j] * x[j];
                }
            }
            new_x[i] = (b[i] - sum) / A[i][i];
        }

        double error = 0.0;
        for (i = 0; i < n; i++) {
            error += fabs(new_x[i] - x[i]);
            x[i] = new_x[i];
        }

        printf("Ітерація №%d: ", k + 1);
        for (i = 0; i < n; i++) {
            printf("%.3lf ", x[i]);
        }
        printf("\n");

        if (error < EPSILON) {
            printf("Ітераційний процес завершився через %d ітерацій.\n", k + 1);
            break;
        }
    }

    if (k == MAX_ITERATIONS) {
        printf("Ітераційний процес перевищив допустимий ліміт ітерацій.\n");
    }

    free(new_x);
}

void gauss_seidel(double **A, double *b, double *x, int n) {
    int i, j, k;
    double sum;

    for (k = 0; k < MAX_ITERATIONS; k++) {
        double *prev_x = malloc(n * sizeof(double)); // Зберігаємо попереднє наближення
        for (i = 0; i < n; i++) {
            prev_x[i] = x[i];
        }

        for (i = 0; i < n; i++) {
            sum = 0.0;
            for (j = 0; j < n; j++) {
                if (j != i) {
                    sum += A[i][j] * x[j];
                }
            }
            x[i] = (b[i] - sum) / A[i][i];
        }

        printf("Ітерація №%d: ", k + 1);
        for (i = 0; i < n; i++) {
            printf("%.3lf ", x[i]);
        }
        printf("\n");

        // Перевірка відносної помилки
        double relative_error = 0.0;
        for (i = 0; i < n; i++) {
            relative_error += fabs((x[i] - prev_x[i]) / x[i]);
        }

        if (relative_error < EPSILON) {
            printf("Процес завершився після %d ітерацій.\n", k + 1);
            break;
        }

        free(prev_x);
    }

    if (k == MAX_ITERATIONS) {
        printf("Перевищено ліміт ітерацій.\n");
    }
}



int main() {
    FILE *file = fopen("input", "r");
    if (file == NULL) {
        printf("Помилка при відкритті файлу.\n");
        return 1;
    }

    int n;
    if (fscanf(file, "%d", &n) != 1) {
        printf("Перевищено максимальний розмір.\n");
        fclose(file);
        return 1;
    }

    double **A1 = malloc(n * sizeof(double *));
    double **A2 = malloc(n * sizeof(double *));
    double *b = malloc(n * sizeof(double));
    double *x1 = malloc(n * sizeof(double));
    double *x2 = malloc(n * sizeof(double));

    int i, j;
    for (i = 0; i < n; i++) {
        A1[i] = malloc(n * sizeof(double));
        A2[i] = malloc(n * sizeof(double));
        for (j = 0; j < n; j++) {
            if (fscanf(file, "%lf", &A1[i][j]) != 1) {
                printf("Помилка при зчитуванні.\n");
                fclose(file);
                return 1;
            }
        }
        if (fscanf(file, "%lf", &b[i]) != 1) {
            printf("Помилка при зчитуванні.\n");
            fclose(file);
            return 1;
        }
        x1[i] = x2[i] = b[i]; // Використовуємо вектор b як початкове наближення для методів
    }

    // Копіюємо A1 в A2
    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            A2[i][j] = A1[i][j];
        }
    }

    fclose(file);

    printf("Метод Якобі:\n");
    jacobi(A1, b, x1, n);

    printf("\nМетод Зейделя:\n");
    gauss_seidel(A2, b, x2, n);

    for (i = 0; i < n; i++) {
        free(A1[i]);
        free(A2[i]);
    }
    free(A1);
    free(A2);
    free(b);
    free(x1);
    free(x2);

    return 0;
}
